package jp.co.powerbeans.vbscore.logic;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import jp.co.powerbeans.docbuild.PBDocmentBuilderFactory;
import jp.co.powerbeans.docbuild.PBDocumentBuilder;
import jp.co.powerbeans.vbscore.Msg;
import jp.co.powerbeans.vbscore.db.DBManager;
import jp.co.powerbeans.vbscore.model.Team;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;


/**
 * ドキュメントマネージャー
 * @author amonden
 */
public class DocumentMgr {

	/**
	 * CSSファイル
	 */
	private static final String CSS = "pbdoc.css";
	/**
	 * tempフォルダ
	 */
	private static final String TEMPLATE_DIR = "template/";
	/**
	 * デフォルト文字コード
	 */
	private static final String CHARSET = "Shift_JIS";

	/**
	 * チーム別ランキング分析を出力
	 * @param shell 親shell
	 * @param team_id チームID
	 * @param table 分析結果Table
	 * @param teamtable チーム名Table
	 */
	public static void outputAnalyzeRanking(Shell shell, int team_id, Table table, Table teamtable) {
		output2TableHtml(shell, "analyzeranking.html",
				Msg.getPlayRanking(),
				table,
				Msg.getTargetGame(),
				teamtable,
				team_id);
	}

	/**
	 * ランキング分析を出力
	 * @param shell 親shell
	 * @param team_id チームID
	 * @param table 分析結果Table
	 */
	public static void outputAnalyzeTeamParGame(Shell shell, int team_id, Table table) {
		output1TableHtml(shell, "teamplaypergame.html",
				Msg.getAnalyzeTeamplayPerGame(),
				table, team_id);
	}
	
	/**
	 * プレイヤー別ランキング分析を出力
	 * @param shell 親shell
	 * @param team_id チームID
	 * @param table 分析結果Table
	 * @param teamtable チーム名Table
	 */
	public static void outputAnalyzePlayer(Shell shell, int team_id, Table table, Table teamtable) {
		output2TableHtml(shell, "analyzeplayer.html",
				Msg.getAnalyzePlayPerPlayer(),
				table,
				Msg.getTargetGame(),
				teamtable,
				team_id);
	}
	
	/**
	 * table を Htmlに出力する
	 * @param shell 親shell
	 * @param template テンプレートファイル
	 * @param title タイトル
	 * @param table 出力元Table
	 * @param team_id チームID
	 */
	private static void output1TableHtml(Shell shell, String template, String title, Table table, int team_id) {
		output2TableHtml(shell, template, title, table, null, null, team_id);
	}
	
	/**
	 * ２つのTableをHtmlに出力する
	 * @param shell 親shell
	 * @param template テンプレートファイル
	 * @param title タイトル
	 * @param table 出力元Table
	 * @param subtitle サブテーブルタイトル
	 * @param subtable サブ出力元Table
	 * @param team_id チームID
	 */
	private static void output2TableHtml(Shell shell, String template, String title, Table table, 
			String subtitle, Table subtable, int team_id) {
		PBDocumentBuilder builder = createDocBuilder(template, CHARSET);

		// 保存用ダイアログを開く
		FileDialog saveDialog = new FileDialog(shell,SWT.SAVE);
		saveDialog.setFilterExtensions(new String[]{"*.html"});
		String output_path = saveDialog.open();
		if (output_path == null) {
			return;
		}
		
		// 1．タイトル
		builder.assign("title", title);
		
		// 2. メインテーブル
		builder.assign("table", createHtmlTable(table));
		
		// 3. サブタイトル
		builder.assign("subtitle", subtitle == null ? "" : subtitle);
		
		// 4. サブテーブル
		if (subtable != null) {
			builder.assign("subtable", createHtmlTable(subtable));
		} else {
		  builder.assign("subtable", "");
    }
		
		// 5. チーム名
		Team team = (Team) DBManager.findByPrimaryKey(Team.class, team_id);
		builder.assign("teamName", team.getTeamName());
		
		// 6.出力
		builder.output(output_path);
		
		// 7. 開く
		try {
			Runtime.getRuntime().exec("explorer " + output_path);
		} catch (IOException e) {
			// TODO 自動生成された catch ブロック
			e.printStackTrace();
		}
	}
	

	private static String createHtmlTable(Table table) {
		StringBuffer buf = new StringBuffer();
		
		buf.append("<table><tr>");
		// 1. 列名出力
		for (int i = 0; i < table.getColumnCount(); i++) {
			TableColumn c = table.getColumn(i);
			buf.append("<th>" + c.getText() + "</th>");
		}
		buf.append("</tr>");
		
		// 2. データ出力
		for (int i = 0; i < table.getItemCount(); i++) {
			TableItem item = table.getItem(i);
			
			// 全列にデータが無ければ終了
			boolean exist = false;
			for (int c = 0; c < table.getColumnCount() && !exist; c++) {
				exist = item.getText(c).length() > 0;
			}
			if (!exist) {
				break;
			}
//			if (table.getColumnCount() > 0 && item.getText(0).length() == 0) {
//				break;
//			}
			
			buf.append("<tr>");
			
			for (int c = 0; c < table.getColumnCount(); c++) {
				buf.append("<td>" + item.getText(c) + "</td>");
			}
			
			buf.append("</tr>");
		}
		buf.append("</table>");
		
		return buf.toString();
	}

	/**
	 * documentBuilderを生成
	 * @param template テンプレートファイル
	 * @param charset 文字コード
	 * @return documentBuilder
	 */
	private static PBDocumentBuilder createDocBuilder(String template, String charset) {
		PBDocumentBuilder builder = PBDocmentBuilderFactory.create(
				PBDocumentBuilder.HTML,	TEMPLATE_DIR + template, charset);
		
		builder.addIncludeFile(CSS);
		assignCommonValue(builder);
		return builder;
	}

	/**
	 * 共通パラーメータを格納
	 * @param builder documentBuilder
	 */
	private static void assignCommonValue(PBDocumentBuilder builder) {
		// date 作成日
		SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
		builder.assign("date", df.format(new Date()));
		
		// footer
		builder.assign("footer", "<span align=center>published by 2006 Akihiko MONDEN <A href='http://amsoft.minidns.net/'>(AMsoft space)</A></span>");
		
		// charset ひとまずShift_JIS固定
		builder.assign("charset", CHARSET);
	}



}
